"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

import zipfile
from dataclasses import dataclass
from functools import lru_cache
from pathlib import Path

from lxml import etree
from lxml.etree import _Element

from arelle import XbrlConst
from arelle.FileSource import FileSource
from arelle.ModelValue import QName, qname
from . import Constants


@dataclass(frozen=True)
class Manifest:
    instances: list[ManifestInstance]
    path: Path
    titlesByLang: dict[str, str]
    tocItems: list[ManifestTocItem]


@dataclass(frozen=True)
class ManifestTocItem:
    extrole: str
    childItems: list[ManifestTocItem]
    itemIn: str
    parent: QName | None
    ref: str
    start: QName | None


@dataclass(frozen=True)
class ManifestInstance:
    id: str
    ixbrlFiles: list[Path]
    preferredFilename: str
    type: str


def _parseManifestTocItems(parentElt: _Element, parentQName: QName | None) -> list[ManifestTocItem]:
    tocItems = []
    for itemElt in parentElt.iterchildren(tag=Constants.qnEdinetManifestItem.clarkNotation):
        childTocItems = []
        for insertElt in itemElt.iterchildren(tag=Constants.qnEdinetManifestInsert.clarkNotation):
            childParentQName = qname(insertElt.attrib.get("parent"), insertElt.nsmap) if insertElt.attrib.get("parent") else None
            childTocItems.extend(_parseManifestTocItems(insertElt, childParentQName))
        tocItems.append(ManifestTocItem(
            extrole=itemElt.attrib.get("extrole", ""),
            childItems=childTocItems,
            parent=parentQName,
            itemIn=itemElt.attrib.get("in", ""),
            ref=itemElt.attrib.get("ref", ""),
            start=qname(itemElt.attrib.get("start"), itemElt.nsmap) if itemElt.attrib.get("start") else None,
        ))
    return tocItems


def _parseManifestDoc(xmlRootElement: _Element, path: Path) -> Manifest:
    instances = []
    titlesByLang = {}
    base = path.parent
    tocElts = list(xmlRootElement.iterchildren(tag=Constants.qnEdinetManifestTocComposition.clarkNotation))
    assert len(tocElts) == 1, 'There should be exactly one tocComposition element in the manifest.'
    for titleElt in tocElts[0].iterchildren(tag=Constants.qnEdinetManifestTitle.clarkNotation):
        lang = titleElt.attrib.get(XbrlConst.qnXmlLang.clarkNotation, "")
        titlesByLang[lang] = titleElt.text.strip() if titleElt.text else ""
    tocItems = _parseManifestTocItems(tocElts[0], None)
    listElts = list(xmlRootElement.iterchildren(tag=Constants.qnEdinetManifestList.clarkNotation))
    assert len(listElts) == 1, 'There should be exactly one list element in the manifest.'
    for instanceElt in listElts[0].iterchildren(tag=Constants.qnEdinetManifestInstance.clarkNotation):
        instanceId = str(instanceElt.attrib.get("id", ""))
        instanceType = str(instanceElt.attrib.get("type", ""))
        preferredFilename = str(instanceElt.attrib.get("preferredFilename", ""))
        ixbrlFiles = []
        for ixbrlElt in instanceElt.iterchildren(tag=Constants.qnEdinetManifestIxbrl.clarkNotation):
            uri = ixbrlElt.text.strip() if ixbrlElt.text is not None else None
            if uri is not None and len(uri) > 0:
                ixbrlFiles.append(base / uri)
        instances.append(ManifestInstance(
            id=instanceId,
            ixbrlFiles=ixbrlFiles,
            preferredFilename=preferredFilename,
            type=instanceType,
        ))
    return Manifest(
        instances=instances,
        path=path,
        titlesByLang=titlesByLang,
        tocItems=tocItems,
    )


@lru_cache(1)
def parseManifests(filesource: FileSource) -> list[Manifest]:
    manifests: list[Manifest] = []
    if filesource.isArchive:
        if filesource.isTaxonomyPackage:
            return manifests
        if filesource.reportPackage is not None:
            return manifests
        for _archiveFile in (filesource.dir or ()):
            if not Path(_archiveFile).stem.startswith('manifest'):
                continue
            assert isinstance(filesource.fs, zipfile.ZipFile), \
                "The EDINET plugin only supports archives in .zip format."
            with filesource.fs.open(_archiveFile) as manifestDoc:
                xmlRootElement = etree.fromstring(manifestDoc.read())
                manifests.append(_parseManifestDoc(xmlRootElement, Path(_archiveFile)))
    elif (dirpath := Path(str(filesource.url))).is_dir():
        for file in dirpath.rglob("*"):
            if not file.is_file():
                continue
            if not file.stem.startswith('manifest'):
                continue
            with open(file, 'rb') as manifestDoc:
                xmlRootElement = etree.fromstring(manifestDoc.read())
                manifests.append(_parseManifestDoc(xmlRootElement, file))
    return manifests



from __future__ import annotations

import html.parser
import os
import urllib.request
import zipfile
from pathlib import Path
from shutil import rmtree

from tests.integration_tests.integration_test_util import get_s3_uri
from tests.integration_tests.scripts.script_util import (
    assert_result,
    parse_args,
    prepare_logfile,
    run_arelle,
    validate_log_file,
)

errors = []
this_file = Path(__file__)
args = parse_args(this_file.stem, "Confirm EBA Tablesets report runs successfully from the command line.")
arelle_command = args.arelle
arelle_offline = args.offline
working_directory = Path(args.working_directory)
test_directory = Path(args.test_directory)
arelle_log_file = prepare_logfile(test_directory, this_file)
samples_zip_path = test_directory / "eba_samples.zip"
samples_directory = test_directory / "eba_samples"
target_path = samples_directory / "DUMMYLEI123456789012_GB_COREP030000_COREPLECON_2021-06-30_20201218154732000.xbrl"
tablesets_report_path = test_directory / "index.html"
sample_report_zip_url = get_s3_uri("ci/packages/eba-samples.zip", version_id="iDJU3nFy6_rQ289k.mosenHjUFrXCmCM")

samples_url = get_s3_uri("ci/packages/eba_samples.zip", version_id="O7uYHbSYmxe_20nBhWWoXMfjGpquNMMj")

print(f"Downloading EBA sample files: {samples_zip_path}")
urllib.request.urlretrieve(samples_url, samples_zip_path)

print(f"Extracting EBA sample files: {samples_directory}")
with zipfile.ZipFile(samples_zip_path, "r") as zip_ref:
    zip_ref.extractall(samples_directory)

print(f"Generating EBA Tablesets report: {tablesets_report_path}")
run_arelle(
    arelle_command,
    plugins=["saveHtmlEBAtables"],
    additional_args=[
        "-f",
        str(target_path),
        "--package",
        str(samples_directory / "TP-Eurofiling2.1.zip"),
        "--package",
        str(samples_directory / "EBA_CRD_IV_XBRL_3.0_Dictionary_3.0.1.0.Errata3.zip"),
        "--package",
        str(samples_directory / "EBA_CRD_IV_XBRL_3.0_Reporting_COREP_FINREP_Frameworks_hotfix.Errata3.zip"),
        "--save-EBA-tablesets",
        str(tablesets_report_path),
    ],
    offline=arelle_offline,
    logFile=arelle_log_file,
)

print(f"Checking for EBA Tablesets report: {tablesets_report_path}")
if not tablesets_report_path.exists():
    errors.append(f'EBA Tablesets report not generated at "{tablesets_report_path}"')

eba_table_files = [
    "eba_tC_00.01.html",
    "eba_tC_26.00.html",
    "eba_tC_27.00.html",
    "eba_tC_28.00.html",
    "eba_tC_29.00.html",
]

eba_tablesets_report_files = [
    test_directory / f for f in ("index.html", "indexCenterLanding.html", "indexFormsFrame.html", *eba_table_files)
]

for report_file in eba_tablesets_report_files:
    if not report_file.exists():
        errors.append(f'EBA Tablesets report file not generated at "{report_file}"')


class ButtonParser(html.parser.HTMLParser):
    def __init__(self, table_files):
        super().__init__()
        self.table_files = table_files
        self.found_buttons = {table_file: False for table_file in table_files}

    def handle_starttag(self, tag, attrs):
        if tag.lower() == "button":
            attrs_dict = dict(attrs)
            onclick = attrs_dict.get("onclick", "")
            if onclick is not None:
                for table_file in self.table_files:
                    expected_onclick = f"javascript:parent.loadContent('{table_file}');"
                    if expected_onclick in onclick:
                        self.found_buttons[table_file] = True


forms_frame_path = test_directory / "indexFormsFrame.html"
print("Checking for proper button elements in indexFormsFrame.html")
with open(forms_frame_path, encoding="utf-8") as fh:
    html_content = fh.read()
    parser = ButtonParser(eba_table_files)
    parser.feed(html_content)
    for table_file, found in parser.found_buttons.items():
        if not found:
            errors.append(f"Button for table {table_file} not found in indexFormsFrame.html")

print(f"Checking for log errors: {arelle_log_file}")
errors += validate_log_file(arelle_log_file)

assert_result(errors)

print("Cleaning up")
rmtree(samples_directory)
os.unlink(samples_zip_path)
for report_file in eba_tablesets_report_files:
    os.unlink(report_file)



"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

import re
from collections import defaultdict
from collections.abc import Iterable
from pathlib import Path
from typing import Any

from arelle.ValidateXbrl import ValidateXbrl
from arelle.typing import TypeGetText
from arelle.utils.PluginHooks import ValidationHook
from arelle.utils.validate.Decorator import validation
from arelle.utils.validate.Validation import Validation
from ..DisclosureSystems import (DISCLOSURE_SYSTEM_EDINET)
from ..FormType import FormType, HTML_EXTENSIONS, IMAGE_EXTENSIONS
from ..Manifest import parseManifests
from ..PluginValidationDataExtension import PluginValidationDataExtension

_: TypeGetText

FILE_COUNT_LIMITS = {
    Path("AttachDoc"): 990,
    Path("AuditDoc"): 990,
    Path("PrivateDoc"): 9_990,
    Path("PublicDoc"): 9_990,
    Path("XBRL") / "AttachDoc": 990,
    Path("XBRL") / "AuditDoc": 990,
    Path("XBRL") / "PrivateDoc": 9_990,
    Path("XBRL") / "PublicDoc": 9_990,
    Path("XBRL"): 99_990,
}

FILENAME_STEM_PATTERN = re.compile(r'[a-zA-Z0-9_-]*')


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0121E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0121E: There is a directory or file that contains more than 31 characters
    or uses characters other than those allowed (alphanumeric characters, '-' and '_').

    Note: Sample instances from EDINET almost always violate this rule based on our
    current interpretation. The exception being files placed outside the XBRL directory,
    i.e. amendment documents. For now, we will only check amendment documents, directory
    names, or other files in unexpected locations.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    uploadContents = pluginData.getUploadContents(val.modelXbrl)
    paths = set(uploadContents.directories | uploadContents.unknownPaths)
    for amendmentPaths in uploadContents.amendmentPaths.values():
        paths.update(amendmentPaths)
    for path in paths:
        if len(str(path.name)) > 31 or not FILENAME_STEM_PATTERN.match(path.stem):
            yield Validation.error(
                codes='EDINET.EC0121E',
                msg=_("There is a directory or file in '%(directory)s' that contains more than 31 characters "
                      "or uses characters other than those allowed (alphanumeric characters, '-' and '_'). "
                      "Directory or file name: '%(basename)s'. "
                      "Please change the file name (or folder name) to within 31 characters and to usable "
                      "characters, and upload again."),
                directory=str(path.parent),
                basename=path.name,
                file=str(path)
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0124E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0124E: There are no empty directories.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    uploadFilepaths = pluginData.getUploadFilepaths(val.modelXbrl)
    emptyDirectories = []
    for path in uploadFilepaths:
        if path.suffix:
            continue
        if not any(path in p.parents for p in uploadFilepaths):
            emptyDirectories.append(str(path))
    for emptyDirectory in emptyDirectories:
        yield Validation.error(
            codes='EDINET.EC0124E',
            msg=_("There is no file directly under '%(emptyDirectory)s'. "
                  "No empty folders. "
                  "Please store the file in the appropriate folder or delete the folder and upload again."),
            emptyDirectory=emptyDirectory,
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0129E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0129E: Limit the number of subfolders to 3 or less from the XBRL directory.
    """
    startingDirectory = 'XBRL'
    if not pluginData.shouldValidateUpload(val):
        return
    uploadFilepaths = pluginData.getUploadFilepaths(val.modelXbrl)
    for path in uploadFilepaths:
        parents = [parent.name for parent in path.parents]
        if startingDirectory in parents:
            parents = parents[:parents.index(startingDirectory)]
        else:
            # TODO: Do we validate amendment subfolders too? These aren't placed beneath the XBRL directory.
            continue
        depth = len(parents)
        if depth > 3:
            yield Validation.error(
                codes='EDINET.EC0129E',
                msg=_("The subordinate directories of %(path)s go up to the level %(depth)s (directories: %(parents)s). "
                      "Please limit the number of subfolders to 3 or less and upload again."),
                path=str(path),
                depth=depth,
                parents=', '.join(f"'{parent}'" for parent in reversed(parents)),
                file=str(path)
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0130E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0130E: File extensions must match the file extensions allowed in Figure 2-1-3 and Figure 2-1-5.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    uploadContents = pluginData.getUploadContents(val.modelXbrl)
    checks = []
    for formType, amendmentPaths in uploadContents.amendmentPaths.items():
        for amendmentPath in amendmentPaths:
            isSubdirectory = amendmentPath.parent.name != formType.value
            checks.append((amendmentPath, True, formType, isSubdirectory))
    for formType, formPaths in uploadContents.forms.items():
        for amendmentPath in formPaths:
            isSubdirectory = amendmentPath.parent.name != formType.value
            checks.append((amendmentPath, False, formType, isSubdirectory))
    for path, isAmendment, formType, isSubdirectory in checks:
        ext = path.suffix
        if len(ext) == 0:
            continue
        validExtensions = formType.getValidExtensions(isAmendment, isSubdirectory)
        if validExtensions is None:
            continue
        if ext not in validExtensions:
            yield Validation.error(
                codes='EDINET.EC0130E',
                msg=_("The file extension '%(ext)s' is not valid at '%(path)s'. "
                      "Valid extensions at this location are: %(validExtensions)s. "
                      "Please change the file extension to a configurable extension and upload it again. "
                      "For information on configurable file extensions, please refer to 'Table 2-1-3 Storable File Formats (1)' "
                      "in the 'Document File Specifications' and 'Table 2-1-5 Storable File Formats (2)' in the "
                      "'Document File Specifications'."),
                ext=ext,
                path=str(path),
                validExtensions=', '.join(f"'{e}'" for e in validExtensions),
                file=str(path)
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0132E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0132E: Store the manifest file directly under the relevant folder.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    uploadContents = pluginData.getUploadContents(val.modelXbrl)
    for formType in (FormType.AUDIT_DOC, FormType.PRIVATE_DOC, FormType.PUBLIC_DOC):
        if formType not in uploadContents.forms:
            continue
        if formType.manifestPath in uploadContents.forms.get(formType, []):
            continue
        yield Validation.error(
            codes='EDINET.EC0132E',
            msg=_("'%(expectedManifestName)s' does not exist in '%(expectedManifestDirectory)s'. "
                  "Please store the manifest file (or cover file) directly under the relevant folder and upload it again. "),
            expectedManifestName=formType.manifestPath.name,
            expectedManifestDirectory=str(formType.manifestPath.parent),
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0183E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0183E: The compressed file size exceeds 55MB.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    size = val.modelXbrl.fileSource.getBytesSize()
    if size is None:
        return  # File size is not available, cannot validate
    if size > 55_000_000:  # Interpretting MB as megabytes (1,000,000 bytes)
        yield Validation.error(
            codes='EDINET.EC0183E',
            msg=_("The compressed file size exceeds 55MB. "
                  "Please compress the file to a size of 55MB or less and upload it again."),
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0188E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0188E: There is an HTML file directly under PublicDoc or PrivateDoc whose first 7 characters are not numbers.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    pattern = re.compile(r'^\d{7}')
    uploadFilepaths = pluginData.getUploadFilepaths(val.modelXbrl)
    docFolders = frozenset({"PublicDoc", "PrivateDoc"})
    for path in uploadFilepaths:
        if path.suffix not in HTML_EXTENSIONS:
            continue
        if path.parent.name not in docFolders:
            continue
        if pattern.match(path.name) is None:
            yield Validation.error(
                codes='EDINET.EC0188E',
                msg=_("There is an html file directly under PublicDoc or PrivateDoc whose first 7 characters are not numbers: '%(path)s'."
                      "Please change the first 7 characters of the file name of the file directly under the folder to numbers "
                      "and upload it again."),
                path=str(path),
                file=str(path),
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0198E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0198E: The number of files in the total submission and directories can not exceed the upper limit.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    fileCounts: dict[Path, int] = defaultdict(int)
    uploadFilepaths = pluginData.getUploadFilepaths(val.modelXbrl)
    for path in uploadFilepaths:
        if len(path.suffix) == 0:
            continue
        for directory in FILE_COUNT_LIMITS.keys():
            if directory in path.parents:
                fileCounts[directory] += 1
                break
    for directory, limit in FILE_COUNT_LIMITS.items():
        actual = fileCounts[directory]
        if actual > limit:
            yield Validation.error(
                codes='EDINET.EC0198E',
                msg=_("The number of files in %(directory)s exceeds the upper limit (%(actual)s > %(limit)s). "
                      "Please reduce the number of files in the folder to below the maximum and try uploading again."),
                directory=str(directory),
                actual="{:,}".format(actual),
                limit="{:,}".format(limit),
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0237E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0237E: The directory or file path to the lowest level exceeds the maximum value (259 characters).
    """
    if not pluginData.shouldValidateUpload(val):
        return
    uploadFilepaths = pluginData.getUploadFilepaths(val.modelXbrl)
    for path in uploadFilepaths:
        if len(str(path)) <= 259:
            continue
        yield Validation.error(
            codes='EDINET.EC0237E',
            msg=_("The directory or file path ('%(path)s') to the lowest level exceeds the maximum value (259 characters). "
                  "Please shorten the absolute path of the folder (or file) "
                  "to 259 characters or less and try uploading again."),
            path=str(path),
            file=str(path),
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC0206E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC0206E: Empty files are not permitted.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    for path, size in pluginData.getUploadFileSizes(val.modelXbrl).items():
        if size > 0:
            continue
        yield Validation.error(
            codes='EDINET.EC0206E',
            msg=_("An empty file exists. "
                  "File name: '%(path)s'. "
                  "Please delete the empty file and upload again."),
            path=str(path),
            file=str(path),
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC1016E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC1016E: The image file is over 300KB.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    for path, size in pluginData.getUploadFileSizes(val.modelXbrl).items():
        if path.suffix not in IMAGE_EXTENSIONS:
            continue
        if size <= 300_000:  # Interpretting KB as kilobytes (1,000 bytes)
            continue
        yield Validation.error(
            codes='EDINET.EC1016E',
            msg=_("The image file is over 300KB. "
                  "File name: '%(path)s'. "
                  "Please create an image file with a size of 300KB or less."),
            path=str(path),
            file=str(path),
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC1020E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC1020E: When writing a DOCTYPE declaration, do not define it multiple times.
    Also, please modify the relevant file so that there is only one html tag, one head tag, and one body tag each.

    Note: Some violations of this rule (such as multiple DOCTYPE declarations) prevent Arelle from parsing
    the XML at all, and thus an XML schema error will be triggered rather than this validation error.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    checkNames = frozenset({'body', 'head', 'html'})
    for modelDocument in val.modelXbrl.urlDocs.values():
        path = Path(modelDocument.uri)
        if path.suffix not in HTML_EXTENSIONS:
            continue
        rootElt = modelDocument.xmlRootElement
        eltCounts = {
            rootElt.qname.localName: 1
        }
        for elt in rootElt.iterdescendants():
            name = elt.qname.localName
            if name not in checkNames:
                continue
            eltCounts[name] = eltCounts.get(name, 0) + 1
            pass
        if any(count > 1 for count in eltCounts.values()):
            yield Validation.error(
                codes='EDINET.EC1020E',
                msg=_("The HTML syntax is incorrect. "
                      "File name: '%(path)s'. "
                      "When writing a DOCTYPE declaration, do not define it multiple times. "
                      "Also, please modify the relevant file so that there is only one html tag, "
                      "one head tag, and one body tag each."),
                path=str(path),
                file=str(path),
            )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_manifest_preferredFilename(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC5804E: The preferredFilename attribute must be set on the instance
    element in the manifest file.

    EDINET.EC5805E: The instance file extension is not ".xbrl". File name: xxx
    Please change the extension of the instance name set in the preferredFilename
    attribute value of the instance element in the manifest file to ".xbrl".

    EDINET.EC5806E: The same instance file name is set multiple times. File name: xxx
    The preferredFilename attribute value of the instance element in the manifest
    file must be unique within the same file.
    """
    if not pluginData.shouldValidateUpload(val):
        return
    manifests = parseManifests(val.modelXbrl.fileSource)
    for manifest in manifests:
        preferredFilenames = set()
        duplicateFilenames = set()
        for instance in manifest.instances:
            if len(instance.preferredFilename) == 0:
                yield Validation.error(
                    codes='EDINET.EC5804E',
                    msg=_("The instance file name is not set. "
                          "Set the instance name as the preferredFilename attribute value "
                          "of the instance element in the manifest file. (manifest: '%(manifest)s', id: %(id)s)"),
                    manifest=str(manifest.path),
                    id=instance.id,
                )
                continue
            preferredFilename = Path(instance.preferredFilename)
            if preferredFilename.suffix != '.xbrl':
                yield Validation.error(
                    codes='EDINET.EC5805E',
                    msg=_("The instance file extension is not '.xbrl'. "
                          "File name: '%(preferredFilename)s'. "
                          "Please change the extension of the instance name set in the "
                          "preferredFilename attribute value of the instance element in "
                          "the manifest file to '.xbrl'. (manifest: '%(manifest)s', id: %(id)s)"),
                    preferredFilename=instance.preferredFilename,
                    manifest=str(manifest.path),
                    id=instance.id,
                )
                continue
            if instance.preferredFilename in preferredFilenames:
                duplicateFilenames.add(instance.preferredFilename)
                continue
            preferredFilenames.add(instance.preferredFilename)
        for duplicateFilename in duplicateFilenames:
            yield Validation.error(
                codes='EDINET.EC5806E',
                msg=_("The same instance file name is set multiple times. "
                      "File name: '%(preferredFilename)s'. "
                      "The preferredFilename attribute value of the instance "
                      "element in the manifest file must be unique within the "
                      "same file. (manifest: '%(manifest)s')"),
                manifest=str(manifest.path),
                preferredFilename=duplicateFilename,
            )



"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

import zipfile
from collections import defaultdict
from dataclasses import dataclass
from functools import lru_cache
from pathlib import Path

from arelle.ModelDocument import Type as ModelDocumentType
from arelle.ModelObject import ModelObject
from arelle.ModelXbrl import ModelXbrl
from arelle.PrototypeDtsObject import LinkPrototype
from arelle.ValidateXbrl import ValidateXbrl
from arelle.typing import TypeGetText
from arelle.utils.PluginData import PluginData
from .FormType import FormType

_: TypeGetText


@dataclass(frozen=True)
class UploadContents:
    amendmentPaths: dict[FormType, frozenset[Path]]
    directories: frozenset[Path]
    forms: dict[FormType, frozenset[Path]]
    unknownPaths: frozenset[Path]


@dataclass
class PluginValidationDataExtension(PluginData):
    _primaryModelXbrl: ModelXbrl | None = None

    # Identity hash for caching.
    def __hash__(self) -> int:
        return id(self)

    @lru_cache(1)
    def shouldValidateUpload(self, val: ValidateXbrl) -> bool:
        """
        Determine if the upload validation should be performed on this model.

        Upload validation should not be performed if the target document is
        not a zipfile.

        Upload validation should only be performed once for the entire package,
        not duplicated for each model. To facilitate this with Arelle's validation
        system which largely prevents referencing other models, we can use `--keepOpen`
        and check if the given model is the first to be loaded.
        :param val: The ValidateXbrl instance with a model to check.
        :return: True if upload validation should be performed, False otherwise.
        """
        modelXbrl = val.modelXbrl
        if modelXbrl == val.testModelXbrl:
            # Not running within a testcase
            if modelXbrl != modelXbrl.modelManager.loadedModelXbrls[0]:
                return False
        if not modelXbrl.fileSource.fs:
            return False  # No stream
        if not isinstance(modelXbrl.fileSource.fs, zipfile.ZipFile):
            return False  # Not a zipfile
        return True

    @lru_cache(1)
    def getFootnoteLinkElements(self, modelXbrl: ModelXbrl) -> list[ModelObject | LinkPrototype]:
        # TODO: Consolidate with similar implementations in EDGAR and FERC
        doc = modelXbrl.modelDocument
        if doc is None:
            return []
        if doc.type in (ModelDocumentType.INLINEXBRL, ModelDocumentType.INLINEXBRLDOCUMENTSET):
            elts = (linkPrototype
                            for linkKey, links in modelXbrl.baseSets.items()
                            for linkPrototype in links
                            if linkPrototype.modelDocument.type in (ModelDocumentType.INLINEXBRL, ModelDocumentType.INLINEXBRLDOCUMENTSET)
                            and linkKey[1] and linkKey[2] and linkKey[3]  # fully specified roles
                            and linkKey[0] != "XBRL-footnotes")
        else:
            rootElt = doc.xmlDocument.getroot()
            elts = rootElt.iterdescendants(tag="{http://www.xbrl.org/2003/linkbase}footnoteLink")
        return [
            elt
            for elt in elts
            if isinstance(elt, (ModelObject, LinkPrototype))
        ]


    def getUploadFileSizes(self, modelXbrl: ModelXbrl) -> dict[Path, int]:
        """
        Get the sizes of files in the upload directory.
        :param modelXbrl: The ModelXbrl instance to get file sizes for.
        :return: A dictionary mapping file paths to their sizes.
        """
        if not self.isUpload(modelXbrl):
            return {}
        assert isinstance(modelXbrl.fileSource.fs, zipfile.ZipFile)
        return {
            Path(i.filename): i.file_size
            for i in modelXbrl.fileSource.fs.infolist()
            if not i.is_dir()
        }

    @lru_cache(1)
    def getUploadContents(self, modelXbrl: ModelXbrl) -> UploadContents:
        uploadFilepaths = self.getUploadFilepaths(modelXbrl)
        amendmentPaths = defaultdict(list)
        unknownPaths = []
        directories = []
        forms = defaultdict(list)
        for path in uploadFilepaths:
            parents = list(reversed([p.name for p in path.parents if len(p.name) > 0]))
            if len(parents) == 0:
                continue
            if parents[0] == 'XBRL':
                if len(parents) > 1:
                    formName = parents[1]
                    formType = FormType.parse(formName)
                    if formType is not None:
                        forms[formType].append(path)
                        continue
            formName = parents[0]
            formType = FormType.parse(formName)
            if formType is not None:
                amendmentPaths[formType].append(path)
                continue
            if len(path.suffix) == 0:
                directories.append(path)
                continue
            unknownPaths.append(path)
        return UploadContents(
            amendmentPaths={k: frozenset(v) for k, v in amendmentPaths.items() if len(v) > 0},
            directories=frozenset(directories),
            forms={k: frozenset(v) for k, v in forms.items() if len(v) > 0},
            unknownPaths=frozenset(unknownPaths)
        )

    @lru_cache(1)
    def getUploadFilepaths(self, modelXbrl: ModelXbrl) -> list[Path]:
        if not self.isUpload(modelXbrl):
            return []
        paths = set()
        assert isinstance(modelXbrl.fileSource.fs, zipfile.ZipFile)
        for name in modelXbrl.fileSource.fs.namelist():
            path = Path(name)
            paths.add(path)
            paths.update(path.parents)
        return sorted(paths)

    @lru_cache(1)
    def isUpload(self, modelXbrl: ModelXbrl) -> bool:
        if not modelXbrl.fileSource.fs or \
                not isinstance(modelXbrl.fileSource.fs, zipfile.ZipFile):
            modelXbrl.warning(
                codes="EDINET.uploadNotValidated",
                msg=_("The target file is not a zip file, so upload validation could not be performed.")
            )
            return False
        return True

    def isStandardTaxonomyUrl(self, uri: str, modelXbrl: ModelXbrl) -> bool:
        return modelXbrl.modelManager.disclosureSystem.hrefValidForDisclosureSystem(uri)



from pathlib import PurePath, Path

from tests.integration_tests.validation.assets import NL_PACKAGES
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig, ConformanceSuiteAssetConfig, AssetSource

config = ConformanceSuiteConfig(
    args=[
        '--disclosureSystem', 'NL-INLINE-2024',
        '--baseTaxonomyValidation', 'none',
        '--testcaseResultsCaptureWarnings',
    ],
    assets=[
        ConformanceSuiteAssetConfig.conformance_suite(
            Path('conformance-suite-2024-sbr-domein-handelsregister.zip'),
            entry_point=Path('conformance-suite-2024-sbr-domein-handelsregister/index.xml'),
            public_download_url='https://www.sbr-nl.nl/sites/default/files/2025-04/conformance-suite-2024-sbr-domein-handelsregister.zip',
            source=AssetSource.S3_PUBLIC,
        ),
        *NL_PACKAGES['NL-INLINE-2024'],
    ],
    expected_additional_testcase_errors={f"conformance-suite-2024-sbr-domein-handelsregister/tests/{s}": val for s, val in {
        'G3-1-3_1/index.xml:TC2_invalid': {
            'scenarioNotUsedInExtensionTaxonomy': 1,  # Also fails 4.2.1.1
        },
        'G3-1-3_2/index.xml:TC2_invalid': {
            'extensionTaxonomyLineItemNotLinkedToAnyHypercube': 1,
            'usableConceptsNotIncludedInDefinitionLink': 1,
        },
        'G3-2-7_1/index.xml:TC1_valid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC2_valid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC3_valid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC4_invalid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC5_invalid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC6_invalid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-2-7_1/index.xml:TC7_invalid': {
            'missingRelevantPlaceholder': 1,
        },
        'G3-5-1_5/index.xml:TC2_invalid': {
            # This is the expected error, but we return two of them, slightly different.
            'imageFormatNotSupported': 1,
        },
        'G3-5-2_3/index.xml:TC2_invalid': {
            'missingLabelForRoleInReportLanguage': 1,
        },
        'G3-5-3_1/index.xml:TC2_invalid': {
            'arelle:ixdsTargetNotDefined': 1,
            'extensionTaxonomyWrongFilesStructure': 2,
            # This test is looking at the usage of the target attribute and does not import the correct taxonomy urls
            'requiredEntryPointNotImported': 1,
            'incorrectKvkTaxonomyVersionUsed': 1,
        },
        'G3-6-3_3/index.xml:TC2_invalid': {
            # Testcase expects only 3.6.3.3, but has a filename that has invalid characters (3.6.3.3)
            # AND {base} with > 20 characters (3.6.3.1)
            'baseComponentInDocumentNameExceedsTwentyCharacters': 1,
        },
        'G3-7-1_1/index.xml:TC2_invalid': {
            'message:valueKvKIdentifier': 13,
            'nonIdenticalIdentifier': 1,
        },
        'G4-1-1_1/index.xml:TC3_invalid': {
            'extensionConceptNoLabel': 1,
        },
        'G4-1-1_1/index.xml:TC4_invalid': {
            'extensionTaxonomyWrongFilesStructure': 1,
        },
        'G4-1-1_1/index.xml:TC5_invalid': {
            'usableConceptsNotIncludedInPresentationLink': 1,
        },
        'G4-1-1_1/index.xml:TC7_invalid': {
            'usableConceptsNotIncludedInPresentationLink': 1,
        },
        'G4-1-2_1/index.xml:TC2_valid': {
            'undefinedLanguageForTextFact': 1,
            'taggedTextFactOnlyInLanguagesOtherThanLanguageOfAReport': 5,
        },
        'G4-1-2_1/index.xml:TC3_invalid': {
            'extensionTaxonomyLineItemNotLinkedToAnyHypercube': 11,
        },
        'G4-1-2_2/index.xml:TC2_invalid': {
            'anchoringRelationshipsForConceptsDefinedInElrContainingDimensionalRelationships': 1,  # Also fails 4.3.2.1
            'incorrectSummationItemArcroleUsed': 1,  # Also fails 4.4.1.1
            # Test imports https://www.nltaxonomie.nl/kvk/2024-03-31/kvk-annual-report-nlgaap-ext.xsd which is the draft taxonomy and not the final
            'requiredEntryPointNotImported': 1,
            'missingRelevantPlaceholder': 1,
            'UsableConceptsNotAppliedByTaggedFacts': 1,  # Also fails 4.4.6.1
            'extensionTaxonomyLineItemNotLinkedToAnyHypercube': 10,
        },
        'G4-2-0_1/index.xml:TC2_invalid': {
            'extensionConceptNoLabel': 1,
        },
        'G4-2-3_1/index.xml:TC2_invalid': {
            'extensionTaxonomyLineItemNotLinkedToAnyHypercube': 1,
            'extensionConceptNoLabel': 1,
            'missingRelevantPlaceholder': 1,
        },
        'G4-4-2_1/index.xml:TC2_invalid': {
            'closedNegativeHypercubeInDefinitionLinkbase': 1,  # Also fails 4.4.2.3
        },
        'G4-4-2_4/index.xml:TC2_invalid': {
            'usableConceptsNotIncludedInDefinitionLink': 1,
        },
        'RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC2_valid': {
            'undefinedLanguageForTextFact': 1,
            'taggedTextFactOnlyInLanguagesOtherThanLanguageOfAReport': 5,
        },
        'RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC4_invalid': {
            'undefinedLanguageForTextFact': 1,
            'taggedTextFactOnlyInLanguagesOtherThanLanguageOfAReport': 5,
        },
        'RTS_Annex_IV_Par_1_G3-1-4_1/index.xml:TC2_invalid': {
            'message:valueKvKIdentifier': 13,
            'nonIdenticalIdentifier': 1,
        },
        'RTS_Annex_IV_Par_1_G3-1-4_2/index.xml:TC2_invalid': {
            'message:valueKvKIdentifier': 13,
        },
        'RTS_Annex_IV_Par_2_G3-1-1_2/index.xml:TC2_invalid': {
            'message:lei-identifier-format': 105,
            'message:valueKvKIdentifierScheme': 105,
        },
        'RTS_Annex_IV_Par_4_3/index.xml:TC4_invalid': {
            'extensionTaxonomyWrongFilesStructure': 1,
        },
        'RTS_Annex_IV_Par_6/index.xml:TC2_valid': {
            'undefinedLanguageForTextFact': 1,
            'taggedTextFactOnlyInLanguagesOtherThanLanguageOfAReport': 5,
        },
        'RTS_Annex_IV_Par_6/index.xml:TC3_invalid': {
            'extensionTaxonomyWrongFilesStructure': 1,
        },
        'RTS_Annex_IV_Par_6/index.xml:TC4_invalid': {
            'undefinedLanguageForTextFact': 1,
            'taggedTextFactOnlyInLanguagesOtherThanLanguageOfAReport': 5,
            'extensionTaxonomyWrongFilesStructure': 1,
        },
        'RTS_Art_6_a/index.xml:TC2_invalid': {
            'UsableConceptsNotAppliedByTaggedFacts': 1,
            'incorrectKvkTaxonomyVersionUsed': 1,
            'message:existsAtLeastOnce_ChamberOfCommerceRegistrationNumber': 1,
            'message:existsAtLeastOnce_FinancialReportingPeriod': 1,
            'message:existsAtLeastOnce_FinancialReportingPeriodEndDate': 1,
            'message:existsAtLeastOnce_LegalEntityLegalForm': 1,
            'message:existsAtLeastOnce_LegalEntityName': 1,
            'message:existsAtLeastOnce_LegalEntityRegisteredOffice': 1,
            'message:existsOnlyOnce_AuditorsReportFinancialStatementsPresent': 1,
            'message:existsOnlyOnce_DocumentAdoptionStatus': 1,
            'message:existsOnlyOnce_FinancialStatementsConsolidated': 1,
        },
    }.items()},
    expected_failure_ids=frozenset([
        # Conformance Suite Errors
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_2/index.xml:TC3_invalid',  # Expects an error code with a preceding double quote. G3-3-1_3 expects the same code without the typo.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_1/index.xml:TC2_invalid',  # Produces: [err:XPTY0004] Variable set Het entity identifier scheme dat bij dit feit hoort MOET het standaard KVK identifier scheme zijn
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_2/index.xml:TC2_invalid',  # Expects fractionElementUsed”.  Note the double quote at the end.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-2_1/index.xml:TC2_invalid',  # Expects inlineXbrlDocumentSetContainsExternalReferences.  Note incorrect "Set".
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-0_2/index.xml:TC2_invalid',  # Expects fractionElementUsed”.  Note the double quote at the end.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-1_1/index.xml:TC2_invalid',  # Expects IncorrectSummationItemArcroleUsed.  Note the capital first character.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_2_G3-1-1_1/index.xml:TC2_invalid',  # Expects NonIdenticalIdentifier instead of nonIdenticalIdentifier (note the cap N)


        # Not Implemented
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G5-1-3_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP Other
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G5-1-3_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP Other
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G5-1-3_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP Other
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G5-1-3_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP Other
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1/index.xml:TC3_invalid',
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_III_Par_1/index.xml:TC2_invalid',
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_III_Par_1/index.xml:TC3_invalid',
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_12_G3-2-4_1/index.xml:TC4_invalid',
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_14_G3-5-1_1/index.xml:TC2_invalid',
    ]),
    info_url='https://www.sbr-nl.nl/sbr-domeinen/handelsregister/uitbreiding-elektronische-deponering-handelsregister',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'validate/NL'}),
    shards=8,
    test_case_result_options='match-all',
)



from pathlib import PurePath, Path

from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig, ConformanceSuiteAssetConfig

config = ConformanceSuiteConfig(
    args=[
        '--baseTaxonomyValidation', 'none',
        '--disclosureSystem', 'EDINET',
        '--testcaseResultsCaptureWarnings',
    ],
    assets=[
        ConformanceSuiteAssetConfig.local_conformance_suite(
            Path('edinet'),
            entry_point=Path('index.xml'),
        ),
    ],
    cache_version_id='cs2wODrDheJqDIm1kEU4Qwk8jwd7DfQu',
    expected_additional_testcase_errors={f"*{s}": val for s, val in {
        # Duplicate errors: Running EDINET validations in a testcase context
        # prevents us from detecting when two models are being validated
        # from the same variation, so `shouldValidateUpload` always returns `True`.
        # This leads to any rule that validates at the package level (rather than the
        # instance level) firing once for each instance. Normal validation runs
        # do not have this issue. We can expect this to be resolved at some point.
        # Until then, if you construct a test case zip from a valid package that has multiple instances
        # (e.g. PublicDoc + AuditDoc) you'll need to add expected additional testcases here
        # for the duplicate errors.
        # TODO: Prevent duplicate runs in testcase context.
        "EC5806E/index.xml:invalid01": {
            # The duplicated instance needed to trigger the duplicated "preferredFilename"
            # error also causes the validation to fire an additional time in the conformance
            # suite context.
            "EDINET.EC5806E": 1,
        }
    }.items()},
    expected_failure_ids=frozenset([]),
    info_url='https://disclosure2.edinet-fsa.go.jp/weee0020.aspx',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'validate/EDINET', 'inlineXbrlDocumentSet'}),
    shards=4,
    test_case_result_options='match-all',
)



from __future__ import annotations

from collections import OrderedDict
from collections.abc import Mapping

import pytest

from arelle.PythonUtil import FrozenDict


class TestFrozenDict:

    def test_init_empty(self) -> None:
        fd = FrozenDict({})
        assert len(fd) == 0
        assert dict(fd) == {}

    def test_getitem(self) -> None:
        fd = FrozenDict({"key": "value", "num": 42, "bool": True})
        assert fd["key"] == "value"
        assert fd["num"] == 42
        assert fd["bool"] is True

    def test_getitem_keyerror(self) -> None:
        fd = FrozenDict({"a": 1})
        with pytest.raises(KeyError):
            fd["missing"]

    def test_iter(self) -> None:
        data = {"x": 10, "y": 20, "z": 30}
        fd = FrozenDict(data)
        keys = list(fd)
        assert keys == ["x", "y", "z"]

    def test_len(self) -> None:
        assert len(FrozenDict({})) == 0
        assert len(FrozenDict({"a": 1})) == 1
        assert len(FrozenDict({"a": 1, "b": 2, "c": 3})) == 3

    def test_repr(self) -> None:
        fd = FrozenDict({"a": 1, "b": 2})
        repr_str = repr(fd)
        assert repr_str == "FrozenDict({'a': 1, 'b': 2})"

    def test_repr_empty(self) -> None:
        fd = FrozenDict({})
        assert repr(fd) == "FrozenDict()"

    def test_eq_with_frozendict(self) -> None:
        fd1 = FrozenDict({"a": 1, "b": 2})
        fd2 = FrozenDict({"a": 1, "b": 2})
        fd3 = FrozenDict({"a": 1, "b": 3})
        fd4 = FrozenDict({"a": 1, "c": 2})

        assert fd1 == fd2
        assert fd1 != fd3
        assert fd1 != fd4

    def test_eq_with_regular_dict(self) -> None:
        fd = FrozenDict({"a": 1, "b": 2})
        regular_dict = {"a": 1, "b": 2}
        different_dict = {"a": 1, "b": 3}
        empty_dict = {}

        assert fd == regular_dict
        assert fd != different_dict
        assert fd != empty_dict

    def test_eq_with_ordered_dict(self) -> None:
        fd = FrozenDict({"a": 1, "b": 2})
        od1 = OrderedDict([("a", 1), ("b", 2)])
        od2 = OrderedDict([("b", 2), ("a", 1)])
        od3 = OrderedDict([("a", 1), ("b", 3)])

        assert fd == od1
        assert fd == od2
        assert fd != od3

    def test_eq_with_non_mapping(self) -> None:
        fd = FrozenDict({"a": 1})
        assert fd != "not a mapping"
        assert fd != 42
        assert fd != ["a", 1]
        assert fd is not None

    def test_hash_consistency(self) -> None:
        fd = FrozenDict({"a": 1, "b": 2})
        hash1 = hash(fd)
        hash2 = hash(fd)
        assert hash1 == hash2

    def test_hash_equal_objects(self) -> None:
        fd1 = FrozenDict({"a": 1, "b": 2})
        fd2 = FrozenDict({"a": 1, "b": 2})
        assert hash(fd1) == hash(fd2)

    def test_hash_different_objects(self) -> None:
        fd1 = FrozenDict({"a": 1, "b": 2})
        fd2 = FrozenDict({"a": 1, "b": 3})
        fd3 = FrozenDict({"a": 1, "c": 2})

        assert hash(fd1) != hash(fd2)
        assert hash(fd1) != hash(fd3)

    def test_hash_order_independence(self) -> None:
        fd1 = FrozenDict({"a": 1, "b": 2})
        fd2 = FrozenDict({"b": 2, "a": 1})
        assert hash(fd1) == hash(fd2)

    def test_immutability(self) -> None:
        original_data = {"a": 1, "b": 2}
        fd = FrozenDict(original_data)

        original_data["c"] = 3
        original_data["a"] = 99

        assert "c" not in fd
        assert fd["a"] == 1
        assert len(fd) == 2

        with pytest.raises(TypeError, match="object does not support item assignment"):
            fd["a"] = 99  # type: ignore[assignment]

    def test_mapping_interface(self) -> None:
        fd = FrozenDict({"a": 1, "b": 2})
        assert isinstance(fd, Mapping)

        assert "a" in fd
        assert "c" not in fd
        assert fd.get("a") == 1
        assert fd.get("c") is None
        assert fd.get("c", "default") == "default"

        keys = list(fd.keys())
        values = list(fd.values())
        items = list(fd.items())

        assert set(keys) == {"a", "b"}
        assert set(values) == {1, 2}
        assert set(items) == {("a", 1), ("b", 2)}

    def test_complex_values(self) -> None:
        complex_data = {"list": [1, 2, 3], "dict": {"nested": "value"}, "tuple": (4, 5, 6), "none": None, "bool": False}
        fd = FrozenDict(complex_data)

        assert fd["list"] == [1, 2, 3]
        assert fd["dict"] == {"nested": "value"}
        assert fd["tuple"] == (4, 5, 6)
        assert fd["none"] is None
        assert fd["bool"] is False

    def test_nested_frozendict(self) -> None:
        inner_fd = FrozenDict({"inner_key": "inner_value"})
        outer_fd = FrozenDict({"outer_key": inner_fd})

        assert outer_fd["outer_key"] == inner_fd
        assert outer_fd["outer_key"]["inner_key"] == "inner_value"

    def test_hashable_keys_only(self) -> None:
        fd = FrozenDict({"string": 1, 42: 2, (1, 2): 3, frozenset([4, 5]): 4})

        assert fd["string"] == 1
        assert fd[42] == 2
        assert fd[(1, 2)] == 3
        assert fd[frozenset([4, 5])] == 4

    def test_dict_methods_not_available(self) -> None:
        fd = FrozenDict({"a": 1})

        assert not hasattr(fd, "clear")
        assert not hasattr(fd, "pop")
        assert not hasattr(fd, "popitem")
        assert not hasattr(fd, "setdefault")
        assert not hasattr(fd, "update")
        assert not hasattr(fd, "__setitem__")
        assert not hasattr(fd, "__delitem__")



from collections.abc import Iterable

import pytest

from arelle.PythonUtil import FrozenOrderedSet, OrderedSet


class TestFrozenOrderedSet:
    def test_empty_initialization(self):
        fos = FrozenOrderedSet()
        assert len(fos) == 0
        assert not fos
        assert list(fos) == []

    def test_initialization_with_list(self):
        fos = FrozenOrderedSet([1, 2, 3])
        assert list(fos) == [1, 2, 3]
        assert len(fos) == 3

    def test_initialization_with_tuple(self):
        fos = FrozenOrderedSet((4, 5, 6))
        assert list(fos) == [4, 5, 6]

    def test_initialization_with_string(self):
        fos = FrozenOrderedSet("abc")
        assert list(fos) == ["a", "b", "c"]

    def test_initialization_with_set(self):
        fos = FrozenOrderedSet({7, 8, 9})
        assert len(fos) == 3
        assert 7 in fos
        assert 8 in fos
        assert 9 in fos

    def test_initialization_with_ordered_set(self):
        fos = FrozenOrderedSet(OrderedSet([7, 8, 9]))
        assert len(fos) == 3
        assert 7 in fos
        assert 8 in fos
        assert 9 in fos

    def test_initialization_with_duplicates(self):
        fos = FrozenOrderedSet([1, 2, 2, 3, 1, 4])
        assert list(fos) == [1, 2, 3, 4]
        assert len(fos) == 4

    def test_contains(self):
        fos = FrozenOrderedSet([1, 2, 3, "a", "b"])

        assert 1 in fos
        assert 2 in fos
        assert 3 in fos

        assert 4 not in fos
        assert None not in fos

    def test_len(self):
        assert len(FrozenOrderedSet()) == 0
        assert len(FrozenOrderedSet([1])) == 1
        assert len(FrozenOrderedSet([1, 2, 3])) == 3
        assert len(FrozenOrderedSet([1, 1, 2, 2, 3])) == 3

    def test_iteration(self):
        fos = FrozenOrderedSet(["first", "second", "third"])
        result = []
        for item in fos:
            result.append(item)
        assert result == ["first", "second", "third"]

    def test_reversed_iteration(self):
        fos = FrozenOrderedSet(["first", "second", "third"])
        result = list(reversed(fos))
        assert result == ["third", "second", "first"]

    def test_repr(self):
        # Empty set
        fos_empty = FrozenOrderedSet()
        assert repr(fos_empty) == "FrozenOrderedSet()"

        # Non-empty set
        fos = FrozenOrderedSet([1, 2, 3])
        assert repr(fos) == "FrozenOrderedSet((1, 2, 3))"

        # With strings
        fos_str = FrozenOrderedSet(["a", "b"])
        assert repr(fos_str) == "FrozenOrderedSet(('a', 'b'))"

    def test_equality_with_frozen_ordered_set(self):
        fos1 = FrozenOrderedSet([1, 2, 3])
        fos2 = FrozenOrderedSet([1, 2, 3])
        fos3 = FrozenOrderedSet([3, 2, 1])
        fos4 = FrozenOrderedSet([1, 2, 3, 4])

        assert fos1 == fos2
        assert fos1 != fos3
        assert fos1 != fos4
        assert FrozenOrderedSet() == FrozenOrderedSet()

    def test_equality_with_ordered_set(self):
        fos = FrozenOrderedSet([1, 2, 3])
        os1 = OrderedSet([1, 2, 3])
        os2 = OrderedSet([3, 2, 1])

        assert fos == os1
        assert fos != os2

    def test_equality_with_other_iterables(self):
        fos = FrozenOrderedSet([1, 2, 3])

        assert fos == {1, 2, 3}
        assert fos == {3, 2, 1}
        assert fos == [1, 2, 3]
        assert fos == [3, 2, 1]
        assert fos == (1, 2, 3)
        assert fos == (3, 2, 1)
        assert fos != [1, 2, 4]
        assert fos != {1, 2, 4}

    def test_equality_with_non_iterables(self):
        fos = FrozenOrderedSet([1, 2, 3])

        assert fos != 42
        assert fos != "123"
        assert fos != None

    def test_hashable(self):
        fos1 = FrozenOrderedSet([1, 2, 3])
        fos2 = FrozenOrderedSet([1, 2, 3])
        fos3 = FrozenOrderedSet([3, 2, 1])

        assert hash(fos1) == hash(fos2)
        assert hash(fos1) != hash(fos3)

        d = {fos1: "value1", fos3: "value3"}
        assert d[fos1] == "value1"
        assert d[fos2] == "value1"
        assert d[fos3] == "value3"

        s = {fos1, fos2, fos3}
        assert len(s) == 2

    def test_hash_consistency(self):
        fos = FrozenOrderedSet([1, 2, 3])
        hash1 = hash(fos)
        hash2 = hash(fos)
        hash3 = hash(fos)

        assert hash1 == hash2
        assert hash2 == hash3

    def test_immutability(self):
        fos = FrozenOrderedSet([1, 2, 3])

        assert not hasattr(fos, "add")
        assert not hasattr(fos, "remove")
        assert not hasattr(fos, "discard")
        assert not hasattr(fos, "pop")
        assert not hasattr(fos, "clear")
        assert not hasattr(fos, "update")

    def test_set_operations_type_compatibility(self):
        fos = FrozenOrderedSet([1, 2, 3])

        assert isinstance(fos, Iterable)

    def test_with_unhashable_types(self):
        with pytest.raises(TypeError):
            FrozenOrderedSet([[1, 2], [3, 4]])

        with pytest.raises(TypeError):
            FrozenOrderedSet([{1: 2}, {3: 4}])

    def test_order_preservation_complex(self):
        fos = FrozenOrderedSet([3, 1, 4, 1, 5, 9, 2, 6, 5])
        expected_order = [3, 1, 4, 5, 9, 2, 6]
        assert list(fos) == expected_order

    def test_boolean_evaluation(self):
        assert not FrozenOrderedSet()
        assert FrozenOrderedSet([1])
        assert FrozenOrderedSet([0])
        assert FrozenOrderedSet([None])

    def test_large_dataset(self):
        large_list = list(range(1000))
        fos = FrozenOrderedSet(large_list)

        assert len(fos) == 1000
        assert list(fos) == large_list
        assert 500 in fos
        assert 1000 not in fos

    def test_nested_iteration(self):
        fos = FrozenOrderedSet(["a", "b", "c"])

        iter1 = iter(fos)
        iter2 = iter(fos)

        assert next(iter1) == "a"
        assert next(iter2) == "a"
        assert next(iter1) == "b"
        assert next(iter2) == "b"

    def test_edge_cases_empty_string_and_zero(self):
        fos = FrozenOrderedSet(["", 0, False, None])

        assert len(fos) == 3
        assert "" in fos
        assert 0 in fos
        assert False in fos
        assert None in fos

        assert list(fos) == ["", 0, None]

    def test_generator_as_input(self):

        def gen():
            yield 1
            yield 2
            yield 3
            yield 2

        fos = FrozenOrderedSet(gen())
        assert list(fos) == [1, 2, 3]
        assert len(fos) == 3

    def test_comparison_with_different_types(self):
        fos = FrozenOrderedSet([1, 2, 3])

        assert fos != "not_iterable_comparison"
        assert fos == frozenset([3, 2, 1])

    def test_index_access(self):
        fos = FrozenOrderedSet([10, 20, 30, 40, 50])

        assert fos[0] == 10
        assert fos[1] == 20
        assert fos[2] == 30
        assert fos[3] == 40
        assert fos[4] == 50

        assert fos[-1] == 50
        assert fos[-2] == 40
        assert fos[-3] == 30
        assert fos[-4] == 20
        assert fos[-5] == 10

    def test_index_access_empty_set(self):
        fos = FrozenOrderedSet()

        with pytest.raises(IndexError):
            fos[0]

        with pytest.raises(IndexError):
            fos[-1]

    def test_index_access_out_of_bounds(self):
        fos = FrozenOrderedSet([1, 2, 3])

        with pytest.raises(IndexError):
            fos[3]

        with pytest.raises(IndexError):
            fos[10]

        with pytest.raises(IndexError):
            fos[-4]

        with pytest.raises(IndexError):
            fos[-10]

    def test_index_access_single_element(self):
        fos = FrozenOrderedSet([42])

        assert fos[0] == 42
        assert fos[-1] == 42

        with pytest.raises(IndexError):
            fos[1]

        with pytest.raises(IndexError):
            fos[-2]

    def test_index_access_with_strings(self):
        fos = FrozenOrderedSet(["apple", "banana", "cherry"])

        assert fos[0] == "apple"
        assert fos[1] == "banana"
        assert fos[2] == "cherry"
        assert fos[-1] == "cherry"
        assert fos[-2] == "banana"
        assert fos[-3] == "apple"

    def test_index_access_with_duplicates_removed(self):
        fos = FrozenOrderedSet([5, 3, 5, 1, 3, 7])

        assert fos[0] == 5
        assert fos[1] == 3
        assert fos[2] == 1
        assert fos[3] == 7

        with pytest.raises(IndexError):
            fos[4]

    def test_index_access_invalid_type(self):
        fos = FrozenOrderedSet([1, 2, 3])

        with pytest.raises(TypeError):
            fos["invalid"]  # type: ignore[assignment]

        with pytest.raises(TypeError):
            fos[1.5]  # type: ignore[assignment]

        with pytest.raises(TypeError):
            fos[None]  # type: ignore[assignment]



"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

from typing import Any

from arelle.ValidateXbrl import ValidateXbrl
from arelle.typing import TypeGetText
from arelle.utils.validate.ValidationPlugin import ValidationPlugin
from .DisclosureSystems import DISCLOSURE_SYSTEM_EDINET
from .PluginValidationDataExtension import PluginValidationDataExtension

_: TypeGetText


class ValidationPluginExtension(ValidationPlugin):
    def newPluginData(self, validateXbrl: ValidateXbrl) -> PluginValidationDataExtension:
        disclosureSystem = validateXbrl.disclosureSystem.name
        if disclosureSystem == DISCLOSURE_SYSTEM_EDINET:
            pass
        else:
            raise ValueError(f'Invalid EDINET disclosure system: {disclosureSystem}')
        return PluginValidationDataExtension(
            self.name,
        )

    def modelXbrlLoadComplete(self, *args: Any, **kwargs: Any) -> None:
        return None



from pathlib import PurePath, Path

from tests.integration_tests.validation.assets import NL_PACKAGES
from tests.integration_tests.validation.conformance_suite_config import ConformanceSuiteConfig, ConformanceSuiteAssetConfig, AssetSource

config = ConformanceSuiteConfig(
    args=[
        '--disclosureSystem', 'NL-INLINE-2024-GAAP-OTHER',
        '--baseTaxonomyValidation', 'none',
        # '--testcaseResultsCaptureWarnings',
    ],
    assets=[
        ConformanceSuiteAssetConfig.conformance_suite(
            Path('conformance-suite-2024-sbr-domein-handelsregister.zip'),
            entry_point=Path('conformance-suite-2024-sbr-domein-handelsregister/index.xml'),
            public_download_url='https://www.sbr-nl.nl/sites/default/files/2025-04/conformance-suite-2024-sbr-domein-handelsregister.zip',
            source=AssetSource.S3_PUBLIC,
        ),
        *NL_PACKAGES['NL-INLINE-2024'],
    ],
    expected_additional_testcase_errors={f"conformance-suite-2024-sbr-domein-handelsregister/tests/{s}": val for s, val in {
        'G5-1-3_1/index.xml:TC1_valid': {
            'noInlineXbrlTags': 1,
        },
        'G5-1-3_1/index.xml:TC2_invalid': {
            'noInlineXbrlTags': 1,
        },
        'G5-1-3_2/index.xml:TC1_valid': {
            'noInlineXbrlTags': 1,
        },
        'G5-1-3_2/index.xml:TC2_invalid': {
            'documentNameDoesNotFollowNamingConvention': 1,
            'noInlineXbrlTags': 1,
            'requiredEntryPointOtherGaapNotReferenced': 1,
        },
    }.items()},
    expected_failure_ids=frozenset([
        # Conformance Suite Errors
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_2/index.xml:TC3_invalid',  # Expects an error code with a preceding double quote. G3-3-1_3 expects the same code without the typo.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_1/index.xml:TC2_invalid',  # Produces: [err:XPTY0004] Variable set Het entity identifier scheme dat bij dit feit hoort MOET het standaard KVK identifier scheme zijn
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_2/index.xml:TC2_invalid',  # Expects fractionElementUsed”.  Note the double quote at the end.
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-2_1/index.xml:TC2_invalid',  # Produces 'EFM.6.03.11' and 'NL.NL-KVK.3.4.2.1.htmlOrXmlBaseUsed'
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_2_G3-1-1_1/index.xml:TC2_invalid',  # Expects NonIdenticalIdentifier instead of nonIdenticalIdentifier (note the cap N)


        # Wont Run
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-2_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-2_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-2_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-2_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-2_2/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-3_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-3_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-3_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-1-3_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-1_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-3_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-3_1/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-3_1/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-3_1/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-4_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-4_2/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-4_2/index.xml:TC3_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-4_2/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC3_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC5_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC6_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-2-7_1/index.xml:TC7_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_2/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-3-1_3/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_3/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_4/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_4/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_5/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-1_5/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-2_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-4-2_1/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_1/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_1/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_1/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_3/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_4/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_4/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_5/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_5/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-1_5/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_1/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_1/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-2_3/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-3_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-3_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-4_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-5-4_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-2_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-2_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-6-3_3/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-7-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G3-7-1_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G6-1-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G6-1-1_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024

        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC5_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC6_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_1/index.xml:TC7_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-1_2/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-2_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-2_1/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-2_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-2_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-2_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC5_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC6_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-1-5_2/index.xml:TC7_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-0_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-0_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-0_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-0_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-1_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-1_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-2_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-2_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-3_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-2-3_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-1_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-1_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-1_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-1_1/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-2_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-3-2_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-1_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-1_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_3/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_3/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_4/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-2_4/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-3_2/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-4_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-4_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-5_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-5_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-5_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-5_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-5_2/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-6_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-6_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/G4-4-6_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS

        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_II_Par_1_RTS_Annex_IV_par_7/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_III_Par_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_III_Par_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_III_Par_1/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_11_G4-2-2_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_11_G4-2-2_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_11_G4-2-2_1/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_12_G3-2-4_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_12_G3-2-4_1/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_12_G3-2-4_1/index.xml:TC3_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_12_G3-2-4_1/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_14_G3-5-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_14_G3-5-1_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_1_G3-1-4_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_1_G3-1-4_1/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_1_G3-1-4_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_1_G3-1-4_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_2_G3-1-1_1/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_2_G3-1-1_2/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_2_G3-1-1_2/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_1/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_1/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_2/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_2/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_3/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_3/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_3/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_3/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_4_3/index.xml:TC5_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_5/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_5/index.xml:TC2_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_5/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_6/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_6/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_6/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_6/index.xml:TC4_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_8_G4-4-5/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_8_G4-4-5/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_8_G4-4-5/index.xml:TC3_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_9_Par_10/index.xml:TC1_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_9_Par_10/index.xml:TC2_valid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Annex_IV_Par_9_Par_10/index.xml:TC3_invalid',  # Must be run with different disclosure system for GAAP/IFRS
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_3/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_3/index.xml:TC2_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_3/index.xml:TC3_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_3/index.xml:TC4_invalid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_6_a/index.xml:TC1_valid',  # Tested in NL-INLINE-2024
        'conformance-suite-2024-sbr-domein-handelsregister/tests/RTS_Art_6_a/index.xml:TC2_invalid',  # Tested in NL-INLINE-2024
    ]),
    info_url='https://www.sbr-nl.nl/sbr-domeinen/handelsregister/uitbreiding-elektronische-deponering-handelsregister',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'validate/NL'}),
    shards=8,
    test_case_result_options='match-all',
)



from __future__ import annotations

import os
import re
import urllib.parse
import urllib.request
from pathlib import Path

import requests

from tests.integration_tests.integration_test_util import get_s3_uri
from tests.integration_tests.scripts.script_util import parse_args, assert_result, prepare_logfile, run_arelle_webserver, validate_log_xml
from tests.integration_tests.validation.assets import ESEF_PACKAGES
from tests.integration_tests.validation.download_assets import download_assets

errors = []
this_file = Path(__file__)
args = parse_args(
    this_file.stem,
    "Confirm ESEF validation runs successfully using the webserver.",
    arelle=False,
)
arelle_command = args.arelle
arelle_offline = args.offline
working_directory = Path(args.working_directory)
test_directory = Path(args.test_directory)
report_zip_path = test_directory / 'TC2_invalid.zip'
arelle_log_file = prepare_logfile(test_directory, this_file)
report_zip_url = get_s3_uri(
    'ci/packages/python_api_validate_esef.zip',
    version_id='U3sEz.B8kjUWw0l6momz87EndK05cxFZ'
)

print(f"Downloading report: {report_zip_path}")
urllib.request.urlretrieve(report_zip_url, report_zip_path)

print("Downloading packages...")
package_assets = {
    package for year in [2017, 2019, 2020, 2021, 2022] for package in ESEF_PACKAGES[year]
}
download_assets(
    assets=package_assets,
    overwrite=False,
    download_and_apply_cache=False,
    download_private=False,
)
package_paths = [str(a.full_local_path) for a in package_assets]

contents = ''
port = 8100
log_xml_bytes = None
with run_arelle_webserver(arelle_command, port) as proc:
    url = f"http://localhost:{port}/rest/xbrl/validation?media=xml"
    url += "&plugins=validate/ESEF"
    url += "&disclosureSystemName=esef"
    url += f"&internetConnectivity={'false' if arelle_offline else 'true'}"
    url += f"&logFile={urllib.parse.quote_plus(str(arelle_log_file))}"
    url += "&packages=" + '|'.join(urllib.parse.quote_plus(str(p)) for p in package_paths)
    url += "&parameters=authority=SE"
    print(f"Validating: {url}")
    with open(report_zip_path, "rb") as f:
        files = {"upload": f}
        response = requests.post(url, files=files)
    response.raise_for_status()
    contents = response.content
    with open(arelle_log_file, 'x') as file:
        log_xml_bytes = contents
        file.write(log_xml_bytes.decode())

if "[info] Activation of plug-in Validate ESMA ESEF successful" not in contents.decode():
    errors.append("Plugin activation failed with response: \n" + contents.decode())

print("Checking log XML for errors...")
errors += validate_log_xml(log_xml_bytes, expected_results={
    'error': {
        re.compile(r'.*\[ESEF\.2\.2\.1\.precisionAttributeUsed] .*'): 1
    },
    'info': {
        re.compile(r'.*\[arelle\.ESEF\.reportPackageSize] The exact report package zipped .*'): 1
    }
})

assert_result(errors)

print("Cleaning up")
try:
    os.unlink(working_directory / 'webserver_validate_esef' / 'TC2_invalid.zip')
except PermissionError as exc:
    print(f"Failed to cleanup test files: {exc}")



from unittest import TestCase
from unittest.mock import MagicMock

from arelle.plugin.validate.ESEF.ESEF_Current.ValidateXbrlFinally import validateCssUrl

class TestValidateCssUrl(TestCase):
    def test_url_function(self) -> None:
        modelXbrl = MagicMock()
        validateCssUrl(
            '* { background: url("http://example.com") }',
            MagicMock(), modelXbrl, MagicMock(), MagicMock(), MagicMock())
        expected = dict(
            level='ERROR',
            codes=('ESEF.3.5.1.inlineXbrlDocumentContainsExternalReferences', 'NL.NL-KVK.3.6.2.1.inlineXbrlDocumentContainsExternalReferences'),
        )
        self.assertLessEqual(expected.items(), modelXbrl.log.call_args.kwargs.items())

    def test_url_token(self) -> None:
        modelXbrl = MagicMock()
        validateCssUrl(
            '* { background: url(http://example.com) }',
            MagicMock(), modelXbrl, MagicMock(), MagicMock(), MagicMock())
        expected = dict(
            level='ERROR',
            codes=('ESEF.3.5.1.inlineXbrlDocumentContainsExternalReferences', 'NL.NL-KVK.3.6.2.1.inlineXbrlDocumentContainsExternalReferences'),
        )
        self.assertLessEqual(expected.items(), modelXbrl.log.call_args.kwargs.items())




import pytest

from arelle.PythonUtil import OrderedSet


class TestOrderedSet:

    def test_init_empty(self):
        os = OrderedSet()
        assert len(os) == 0
        assert list(os) == []
        assert bool(os) is False

    def test_init_with_iterable(self):
        os = OrderedSet([1, 2, 3, 2, 1])
        assert len(os) == 3
        assert list(os) == [1, 2, 3]

    def test_add_duplicate_order(self):
        os = OrderedSet()
        os.add(1)
        os.add(2)
        os.add(1)
        assert list(os) == [1, 2]
        assert len(os) == 2

    def test_contains(self):
        os = OrderedSet([1, 2, 3])
        assert 1 in os
        assert 2 in os
        assert 3 in os
        assert 4 not in os
        assert "1" not in os

    def test_update(self):
        os = OrderedSet([1, 2])
        os.update([3, 4, 2, 5])
        assert list(os) == [1, 2, 3, 4, 5]

    def test_update_with_string(self):
        os = OrderedSet(['a', 'b'])
        os.update("bcd")
        assert list(os) == ['a', 'b', 'c', 'd']

    def test_discard(self):
        os = OrderedSet([1, 2, 3, 4])
        os.discard(2)
        assert list(os) == [1, 3, 4]

        # Discarding non-existent element should not raise error
        non_existent_element = 10
        os.discard(non_existent_element)
        assert list(os) == [1, 3, 4]

    def test_discard_from_empty(self):
        os = OrderedSet()
        os.discard(1)
        assert len(os) == 0

    def test_pop_last(self):
        os = OrderedSet([1, 2, 3, 4])
        result = os.pop()
        assert result == 4
        assert list(os) == [1, 2, 3]

    def test_pop_first(self):
        os = OrderedSet([1, 2, 3, 4])
        result = os.pop(last=False)
        assert result == 1
        assert list(os) == [2, 3, 4]

    def test_pop_empty_set(self):
        os = OrderedSet()
        with pytest.raises(KeyError, match="set is empty"):
            os.pop()

    def test_iteration(self):
        os = OrderedSet([1, 2, 3])
        result = []
        for item in os:
            result.append(item)
        assert result == [1, 2, 3]

    def test_reversed_iteration(self):
        os = OrderedSet([1, 2, 3])
        result = list(reversed(os))
        assert result == [3, 2, 1]

    def test_len(self):
        os = OrderedSet()
        assert len(os) == 0

        os.add(1)
        assert len(os) == 1

        os.add(2)
        assert len(os) == 2

        os.add(1)  # Duplicate
        assert len(os) == 2

    def test_bool(self):
        os = OrderedSet()
        assert bool(os) is False

        os.add(1)
        assert bool(os) is True

    def test_repr_empty(self):
        os = OrderedSet()
        assert repr(os) == "OrderedSet()"

    def test_repr_with_elements(self):
        os = OrderedSet([1, 2, 3])
        assert repr(os) == "OrderedSet([1, 2, 3])"

    def test_eq_with_ordered_set(self):
        os1 = OrderedSet([1, 2, 3])
        os2 = OrderedSet([1, 2, 3])
        os3 = OrderedSet([3, 2, 1])
        os4 = OrderedSet([1, 2])

        assert os1 == os2
        assert os1 != os3
        assert os1 != os4

    def test_eq_with_regular_set(self):
        os = OrderedSet([1, 2, 3])
        s = {1, 2, 3}
        s_different = {1, 2}

        assert os == s
        assert os != s_different

    def test_eq_with_list(self):
        os = OrderedSet([1, 2, 3])
        lst = [1, 2, 3]

        assert os == lst

    def test_eq_with_non_iterable(self):
        os = OrderedSet([1, 2, 3])
        assert os != 42
        assert os != "hello"

    def test_order_preservation(self):
        os = OrderedSet()
        items = [5, 1, 3, 2, 4]
        for item in items:
            os.add(item)
        assert list(os) == items

    def test_order_preservation_with_duplicates(self):
        os = OrderedSet()
        os.add(1)
        os.add(2)
        os.add(3)
        os.add(2)
        assert list(os) == [1, 2, 3]

    def test_complex_operations(self):
        os = OrderedSet([1, 2, 3, 4, 5])

        os.discard(3)
        os.discard(5)
        assert list(os) == [1, 2, 4]

        os.add(6)
        os.add(0)
        assert list(os) == [1, 2, 4, 6, 0]

        os.update([7, 2, 8])
        assert list(os) == [1, 2, 4, 6, 0, 7, 8]

        last = os.pop()
        first = os.pop(last=False)
        assert last == 8
        assert first == 1
        assert list(os) == [2, 4, 6, 0, 7]

    def test_empty_set_operations(self):
        os = OrderedSet()

        assert list(os) == []
        assert list(reversed(os)) == []

        os.update([])
        assert len(os) == 0

    def test_single_element_operations(self):
        os = OrderedSet([42])

        assert len(os) == 1
        assert 42 in os
        assert list(os) == [42]
        assert list(reversed(os)) == [42]

        result = os.pop()
        assert result == 42
        assert len(os) == 0

    def test_unhashable_types_not_supported(self):
        os = OrderedSet()
        with pytest.raises(TypeError):
            os.add([1, 2, 3])

        with pytest.raises(TypeError):
            os.add({1: 2})

    def test_index_access_basic(self):
        os = OrderedSet([1, 2, 3, 4, 5])

        assert os[0] == 1
        assert os[1] == 2
        assert os[2] == 3
        assert os[3] == 4
        assert os[4] == 5

        assert os[-1] == 5
        assert os[-2] == 4
        assert os[-3] == 3
        assert os[-4] == 2
        assert os[-5] == 1

    def test_index_access_out_of_bounds(self):
        os = OrderedSet([1, 2, 3])

        with pytest.raises(IndexError):
            os[3]
        with pytest.raises(IndexError):
            os[10]

        with pytest.raises(IndexError):
            os[-4]
        with pytest.raises(IndexError):
            os[-10]

    def test_index_access_empty_set(self):
        os = OrderedSet()

        with pytest.raises(IndexError):
            os[0]
        with pytest.raises(IndexError):
            os[-1]

    def test_index_access_single_element(self):
        os = OrderedSet([42])

        assert os[0] == 42
        assert os[-1] == 42

        with pytest.raises(IndexError):
            os[1]
        with pytest.raises(IndexError):
            os[-2]

    def test_index_access_after_modifications(self):
        os = OrderedSet([1, 2, 3, 4])

        os.discard(2)
        assert os[0] == 1
        assert os[1] == 3
        assert os[2] == 4

        os.add(5)
        assert os[3] == 5

        os.pop()
        assert len(os) == 3
        with pytest.raises(IndexError):
            os[3]



"""
See COPYRIGHT.md for copyright information.
"""
from arelle.ModelValue import qname

qnEdinetManifestInsert = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}insert")
qnEdinetManifestInstance = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}instance")
qnEdinetManifestItem = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}item")
qnEdinetManifestIxbrl = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}ixbrl")
qnEdinetManifestList = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}list")
qnEdinetManifestTitle = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}title")
qnEdinetManifestTocComposition = qname("{http://disclosure.edinet-fsa.go.jp/2013/manifest}tocComposition")



"""
See COPYRIGHT.md for copyright information.
"""
from __future__ import annotations

import itertools
from collections.abc import Iterable
from typing import Any

from arelle import XbrlConst
from arelle.ValidateXbrl import ValidateXbrl
from arelle.typing import TypeGetText
from arelle.utils.PluginHooks import ValidationHook
from arelle.utils.validate.Decorator import validation
from arelle.utils.validate.Validation import Validation
from ..DisclosureSystems import (DISCLOSURE_SYSTEM_EDINET)
from ..PluginValidationDataExtension import PluginValidationDataExtension

_: TypeGetText


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC5002E(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC5002E: A unit other than number of shares (xbrli:shares) has been set for the
    Number of Shares (xbrli:sharesItemType) item '{xxx}yyy'.
    Please check the units and enter the correct information.

    Similar to "xbrl.4.8.2:sharesFactUnit-notSharesMeasure" and "xbrl.4.8.2:sharesFactUnit-notSingleMeasure"
    TODO: Consolidate this rule with the above two rules if possible.
    """
    errorFacts = []
    for fact in val.modelXbrl.facts:
        concept = fact.concept
        if not concept.isShares:
            continue
        unit = fact.unit
        measures = unit.measures
        if (
                not measures or
                len(measures[0]) != 1 or
                len(measures[1]) != 0 or
                measures[0][0] != XbrlConst.qnXbrliShares
        ):
            errorFacts.append(fact)
    for fact in errorFacts:
        yield Validation.error(
            codes='EDINET.EC5002E',
            msg=_("A unit other than number of shares (xbrli:shares) has been set for "
                  "the Number of Shares (xbrli:sharesItemType) item '%(qname)s'. "
                  "Please check the units and enter the correct information."),
            qname=fact.qname.clarkNotation,
            modelObject=fact,
        )


@validation(
    hook=ValidationHook.XBRL_FINALLY,
    disclosureSystems=[DISCLOSURE_SYSTEM_EDINET],
)
def rule_EC8033W(
        pluginData: PluginValidationDataExtension,
        val: ValidateXbrl,
        *args: Any,
        **kwargs: Any,
) -> Iterable[Validation]:
    """
    EDINET.EC8033W: The startDate of a context whose context ID starts with
    "CurrentYear" is not set to a date earlier than the endDate of a context
    whose context ID starts with "Prior1Year".
    """
    priorYearContexts = [
        context
        for contextId, context in val.modelXbrl.contexts.items()
        if contextId.startswith('Prior1Year')
           and context.endDatetime is not None
           and context.isStartEndPeriod
    ]
    currentYearContexts = [
        context
        for contextId, context in val.modelXbrl.contexts.items()
        if contextId.startswith('CurrentYear')
           and context.startDatetime is not None
           and context.isStartEndPeriod
    ]
    for priorYearContext, currentYearContext in itertools.product(priorYearContexts, currentYearContexts):
        if priorYearContext.endDatetime > currentYearContext.startDatetime:
            yield Validation.warning(
                codes='EDINET.EC8033W',
                msg=_("The startDate element of the current year context (id=%(currentYearContextId)s) is "
                      "set to a date that is earlier than the endDate element of the prior year context "
                      "(id=%(priorYearContextId)s). Please check the corresponding context ID "
                      "%(currentYearContextId)s and %(priorYearContextId)s. Set the startDate element of "
                      "context ID %(currentYearContextId)s to a date that is later than or equal to the "
                      "endDate element of context ID %(priorYearContextId)s."),
                currentYearContextId=currentYearContext.id,
                priorYearContextId=priorYearContext.id,
                modelObject=(priorYearContext, currentYearContext),
            )
